import static java.lang.Math.*;


public class atomFunc{
    static vectorFunc v;
    
    public atomFunc()
    {
        v = new vectorFunc();
    }
    
    public static atom zeroforce(atom i)    // reset force accumulator
    {
        atom j = new atom(i);
        j.force = new vector(0,0);    
        return j;
    }
    
    public static vector separate(atom i)        // return randomized separation force adjusted for mass
    {
        double rx = (double)Math.random()+0.5d;
        double ry = (double)Math.random()-0.5d;
        return v.scale(new vector(rx, ry), rx+ry*1000*i.mass);
    }
    
    public static atom addforce(atom i, vector f)    // accumulate force
    {
        atom j = new atom(i);
        j.force = v.add(j.force, f);
        return j;
    }
    
    public static atom subforce(atom i, vector f)    // negative accumulate force
    {
        atom j = new atom(i);
        j.force = v.sub(j.force, f);
        return j;
    }
    
    public static vector repulseSQ(atom i, atom j, double restlen, double distcentSQ, double damp, double deep)    // return self repulsion spring+damper forces
    {
        vector link = v.sub(i.pos, j.pos);
        double multSQ = pow(restlen,2) /  distcentSQ;
        double comp = (i.mass*j.mass)/(i.mass+j.mass);
        
        vector spring = v.scale(v.sub(v.scale(link, multSQ), link), comp*deep); // dist=distance surface to surface. (how deep) K=5000
        vector damper = v.scale((v.sub(j.vel, i.vel)), comp*damp);            // D=damping
        return v.add(spring, damper);
    }
    
    public static vector invRepulseSQ(atom i, vector pos, double restlen, double distcentSQ, double damp, double deep)    // return self repulsion spring+damper forces
    {
        vector link = v.sub(i.pos, pos);
        double multSQ = pow(restlen,2) /  distcentSQ;
        double comp = i.mass;
        
        vector spring = v.scale(v.sub(v.scale(link, multSQ), link), comp*deep); // dist=distance surface to surface. (how deep) K=5000
        vector damper = v.scale( i.vel, comp*damp);            // D=damping
        return v.sub(spring, damper);
    }
    
    public static atom gravity(atom i, vector grav)        // apply gravity on atom
    {
        atom j = new atom(i);
        j.force = v.add(j.force, v.scale(grav, j.mass));
        return j;
    }
    
    public static atom integrate(atom i, double dt)        // integrate system
    {
        atom j = new atom(i);
        j.vel = v.add(j.vel, v.scale(v.scale(j.force, j.imass), dt));     // v = (f/m) *dt   or   v = a*dt  or  v = integrated v
        j.pos = v.add(j.pos, v.scale(j.vel, dt));                           // x += vel *dt
        
        return j;
    }
    
    
    public static atom terminalforces(atom i)        // limit maximum forces in atom accumulator
    {
        atom j = new atom(i);
        j.force = v.limit(j.force, 8000000);
        return j;
    }
    
    public static atom terminalvelocity(atom i)        // limit maximum velocity
    {
        atom j = new atom(i);
        j.vel = v.limit(j.vel, 10000);
        return j;
    }
    
    public static atom velocitydrag(atom i, double drag)        // apply scaling factor to velocity, simulate drag
    {
        atom j = new atom(i);
        j.vel = v.scale(j.vel, drag);
        return j;
    }
    
    //spring
    
    public static atom linkWith(atom atomSrc, int atomNum, double length)
    {
        if (atomSrc.linkQty >= atomSrc.maxLink-1)
            return atomSrc;
        atom nuatom = new atom(atomSrc);
        nuatom.linkedTo[nuatom.linkQty] = atomNum;
        nuatom.breaklimit[nuatom.linkQty] = 30000;
        nuatom.rigidity[nuatom.linkQty] = 300;
        nuatom.lenght[nuatom.linkQty] = length;
        nuatom.linkQty++;
        return nuatom;
    }
    
    
    //correct these
    public static atom springForceSQ(atom atomSrc, atom[] atomLst)
    {
        if (atomSrc.linkQty == 0)
            return atomSrc;
        atom nuatom = new atom(atomSrc);
        atom atomDest;
        for (int i=0; i<atomSrc.linkQty;i++)
        {
            atomDest = atomLst[atomSrc.linkedTo[i]];
            double distCentersSQ = v.distSQ(atomSrc.pos, atomDest.pos);  // returns SQdistance of atoms centers
            if (distCentersSQ == 0)
            {
                distCentersSQ = 0.1d;                     // case distance = 0, make it work. (could delete spring too or break)
            }
            nuatom = addforce(nuatom, repulseSQ(atomSrc, atomDest, atomSrc.lenght[i], distCentersSQ, atomSrc.rigidity[i],250000));
        }
        return nuatom;
    }
};
